/*****************************************************
CVAVR V2.05
Chip type               : ATtiny13A
AVR Core Clock frequency: 9,6 MHz
Project                 : sinewave inverter (ir2103|ir2304|ir2109)
                        : 18kHz/50Hz
Date                    : June, 2016
*****************************************************/
#include <tiny13a.h>

#define r_up      30000
#define r_lo      2000
#define Vbat_adc  (110UL*(r_up+r_lo)/r_lo) // 1595
#define adc_12V   (1200UL*0xff/Vbat_adc)   //191

unsigned char c, ADC1, tmr;
signed int corr=-0xff/2,crv=-0xff/2;

/*
#define s_step 188
flash unsigned char s_table [s_step/2]=
{0,4,8,12,17,21,25,29,34,38,42,46,50,55,59,63,67,71,75,79,83,
88,92,95,99,103,107,111,115,119,123,126,130,134,137,141,144,148,151,155,158,161,
165,168,171,174,177,181,184,186,189,192,195,198,200,203,206,208,211,213,215,218,220,
222,224,226,228,230,232,233,235,237,238,240,241,243,244,245,246,247,249,249,250,251,
252,253,253,254,254,255,255,255,255,255};
*/
#define s_step 360
flash unsigned char s_table [s_step/2]=
{0,2,4,6,8,11,13,15,17,20,22,24,26,28,31,33,35,37,40,42,44,
46,48,51,53,55,57,59,61,64,66,68,70,72,74,76,79,81,83,85,87,89,
91,93,95,97,100,102,104,106,108,110,112,114,116,118,120,122,124,126,127,129,131,
133,135,137,139,141,143,144,146,148,150,152,154,155,157,159,161,162,164,166,167,169,
171,172,174,176,177,179,181,182,184,185,187,188,190,191,193,194,196,197,198,200,201,
203,204,205,207,208,209,210,212,213,214,215,217,218,219,220,221,222,223,224,226,227,
228,229,230,231,232,232,233,234,235,236,237,238,238,239,240,241,242,242,243,244,244,
245,246,246,247,247,248,248,249,249,250,250,251,251,252,252,252,253,253,253,254,254,
254,254,255,255,255,255,255,255,255,255,255,255};

void t0_cfg()
{
 TCCR0A = 1<<COM0A1|1<<WGM00;
 TCCR0B = 1<<CS00;
 TIMSK0 = 1<<OCIE0A;
}
void gpio_cfg()
{
 PORTB = 0b11101000;
 DDRB  = 0b11110011;
}
void adc_cfg()
{
 ADMUX  = 1<<REFS0|1<<ADLAR|1;
 ADCSRA = 1<<ADEN|1<<ADSC|7;
}
void exiti_cfg()
{
 GIMSK|=1<<PCIE;
 PCMSK|=1<<PCINT3;
}

void bat_detect()
{
 static bit bt=0;

 if (ADC1>1050UL*0xff/Vbat_adc) tmr=0xff;

 if ((ADC1<900UL*0xff/Vbat_adc||!tmr) && !bt)
 {TCCR0A=TIMSK0=0;bt=c=1;}
 else if(ADC1>1200UL*0xff/Vbat_adc && bt)
 {TIMSK0 = 1<<OCIE0A;TCCR0A=1<<COM0A1|1<<WGM00;bt=0;}
}
void adc_works()
{
 static unsigned int dtmp = 0;
 static unsigned char cnt  = 0;

 if(!ADCSRA.ADSC)
 {
  cnt++;
  dtmp+=ADCH;

  if(cnt>0xff/2)
  {
   signed int tmp;
   ADC1 = dtmp>>7;
   cnt  = 1;
   dtmp = ADCH;
   if(ADC1>adc_12V) tmp = (signed int)adc_12V-ADC1;
   else             tmp = adc_12V-ADC1;
   #asm("cli")
   crv=tmp;
   #asm("sei")
   bat_detect();
  }
  ADCSRA |= 1<<ADSC;
 }
}

interrupt [TIM0_COMPA] void timer0_compa_isr(void)
{
 static bit dir;
 signed int tmp;

 tmp = (signed int)s_table[c]+corr;
 if(tmp>250)tmp=250;
 else if(tmp<0)tmp=0;

 //PORTB.4^=1;

 if(!c)
 {
  dir=c=1;
  TCCR0A = 1<<COM0B1|1<<WGM00;
  TIMSK0 = 1<<OCIE0B;
  corr=crv;
 }
 else if((dir)&&(c<s_step/2-1)){OCR0A=(unsigned char)tmp; c++;}
 else {c--; OCR0A=(unsigned char)tmp; dir=0;}
}
interrupt [TIM0_COMPB] void timer0_compb_isr(void)
{
 static bit dir;
 signed int tmp;

 tmp = (signed int)s_table[c]+corr;
 if(tmp>250)tmp=250;
 else if(tmp<0)tmp=0;

 if(!c)
 {
  dir=1;c=1;
  TCCR0A = 1<<COM0A1|1<<WGM00;
  TIMSK0 = 1<<OCIE0A;
  if(tmr)tmr--;
 }
 else if(dir && c<s_step/2-1){OCR0B=(unsigned char)tmp; c++;}
 else {c--; OCR0B=(unsigned char)tmp; dir=0;}
}
interrupt [PC_INT0] void pin_change_isr(void)
{
 unsigned int i;
 unsigned char k;

 if(!PINB.3)
 {
  TCCR0A = 0;
  for(k=0xff;k;k--)for(i=0xffff;i;i--)#asm("wdr");
  TCCR0A = 1<<COM0A1|1<<WGM00;
 }
}

void main(void)
{
gpio_cfg();
adc_cfg();
exiti_cfg();
t0_cfg();


// Crystal Oscillator division factor: 1
#pragma optsize-
CLKPR=0x80;
CLKPR=0x00;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

#pragma optsize-
WDTCR=0x39;
WDTCR=0x69;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

#asm("sei")

while (1)
      {

       adc_works();
       #asm("wdr")
      }
}
